//
// Programmer:    Jane Rivera <mejane@ccrma.stanford.edu>
// Modified by:   Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed May 22 20:52:28 PDT 1996
// Last Modified: Wed May 22 20:52:42 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/blues.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "blues-main.cc"
#include "psr-520.h"


EventBuffer eventBuffer(100);		// EventBuffer holding up to 100 events

const int NumKeys = 48;			// Number of notes in the bass line
const int NumCompVoices = 5; 		// Number of notes in the piano comp
Note comp[NumCompVoices];		// Note object for each note in comp 
Note bass(CH_2);			// Note object for bassline
Note improv(CH_3);			// Note object for improv line
NoteLoop bassline;			// Notes for the bassline

// Intervals of three different comp chords (each has 5 notes):
int compChords[3][NumCompVoices] = { 12, 10, 16, 21, 26,
                                     04, 10, 21, 26, 31,
                                     04, 10, 14, 31, 36    };

int BluesScale[6] = {0, 3, 5, 6, 7, 10};   // blues scale degrees for improv
int progression[12]= {0,5,0,0,5,5,0,0,7,5,0,7}; // chords roots each measure


/***************** Beginning of user functions ****************************/

class BassParameters {
   public:
      int status;
      int beat;
      BassParameters(void) { status = 0; beat = 0; }
};

BassParameters bassParameters;

#define PARAMS ((BassParameters*)((FunctionNote*)note)->parameters())

static void bassnotesFunction(Note* note) {
   if (PARAMS->status == 0) {
      note->off();
   } else {
      PARAMS->beat = (PARAMS->beat + 1) % NumKeys;
      bass.play(bassline.nextKey() + 36 + transposition, 
                (int)(bassline.nextVel() * drum.p1ps()));
      note->setOnTime((int)(note->getOnTime()+bassline.nextDur()*
                            (1-drum.p3ps()+0.5)));
   }
}


/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "_____________________________________________________________________ \n"
   << "| Description:                                                       |\n"
   << "|                                                                    |\n"
   << "| <Jane Rivera><Music120><FinalProject><March1996>                   |\n"
   << "|                                                                    |\n"
   << "| 3 channels worth of Blues: a ground bassline, \"comp\" chords        |\n"
   << "| (stick 1 trigger) in three inversions (x-axis) through three       |\n"
   << "| octaves (y-axis), and a blues scale improv (stick 2 pollresponse)  |\n"
   << "| mapped horizontally with three vertical octave shifts.  Bass &     |\n"
   << "| blues scale improv velocity mapped to pots 1 & 2 respectively.     |\n"
   << "| Tempo of bass mapped to pot 3.  Velocity of comp chords controlled |\n"
   << "| by whack.                                                          |\n"
   << "|                                                                    |\n"
   << "----------------------------------------------------------------------\n"
   ;
}

void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1: Trigger comp chords                                 \n"
   << ">    x1t: 3 inversions of a comp chord.                        \n"
   << ">    y1t: 3 octaves for the comp chord.                        \n"
   << ">    w1t: Loudness of the comp chord.                          \n"
   << ">    x1p:                                                      \n"
   << ">    y1p:                                                      \n"
   << ">    z1p: duration of the comp chord.                          \n"
   << "> Baton 2: Improv line.                                        \n"
   << ">    x2t:                                                      \n"
   << ">    y2t:                                                      \n"
   << ">    w2t:                                                      \n"
   << ">    x2p: Blues scale steps (6) for imrov.                     \n"
   << ">    y2p: 3 Octaves of blues scale improv.                     \n"
   << ">    z2p: On/Off status of the improv line.                    \n"
   << ">                                                              \n"
   << ">    p1p: Loudness of bass line.                               \n"
   << ">    p2p: Loudness of improv line.                             \n"
   << ">    p3p: Tempo of bass line.                                  \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}


/*---------------- Initialization functions ------------------------------*/

void makeBassLine(void) {
   // 12-bar bassline for progression of: I,IV,I,I, IV,IV,I,I, V,IV,I,V
   char keys[NumKeys] = {
        0,  0,  7,  6,
        5,  5,  9, 10,
       12, 12, 10,  9,
        7,  0,  3,  4,
        5,  5, 12, 11,
       12, 11, 10,  3,
        0,  0,  7,  6,
        7,  4,  5,  6,
        7,  7, 14,  6,
        5,  5, 12,  1,
        0,  4, 10,  9,
        7, -5, -2, -1      
   };      
   const int numVels = 47;    // Off by one for interesting effect
   char vels[numVels] = {
       127,  90, 110,  95,
       100,  80,  95,  75,
       120,  90, 110,  95,
       100,  80,  95,  75,
       120,  90, 110,  95,
        80, 100,  75,  95,
       120,  90, 110,  95,
       100,  80,  95,  75,
       120,  90, 110,  95,
        80,  80, 127, 110,
       120,  90, 110,  95,
        80, 127, 127   		// , 127
   };
   const int numDurs = 1;
   long durs[numDurs] = {500};

   bassline.setKeys(keys, NumKeys);
   bassline.setVels(vels, numVels);
   bassline.setDurs(durs, numDurs);
}


void initialization(void) { 
   description(); 
   drum.setPollRate(20);
   eventBuffer.setPollRate(10);
   for (int i=1; i<NumCompVoices; i++) {
      comp[0].setLinkTo(&comp[i]);
   }
   comp[0].sendTimbre(PSR_BRIGHT_ACOUSTIC_PIANO);
   bass.sendTimbre(PSR_ELECTRIC_BASS_PICK);
   improv.sendTimbre(PSR_TRUMPET);

   midi.cont(CH_1, 7, 127);    
   midi.cont(CH_2, 7, 127);
   midi.cont(CH_3, 7, 127);

   makeBassLine();
}


/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { 
   drum.checkPollAll();
   eventBuffer.checkPoll();
}

/*---------------- Trigger response functions ----------------------------*/

void turnBassOn(void) {
   bassParameters.status = ON;
   FunctionNote tempnote;
   tempnote.setFunction(&bassnotesFunction);
   tempnote.setParameters(&bassParameters);
   tempnote.setOnOff(t_time, 0);
   eventBuffer.insert(&tempnote);
}

void stick1trig(void) { 
   if (bassParameters.status == OFF) {
      turnBassOn();
   } else {
      for (int i=0; i<NumCompVoices; i++) {
         int note = 48 + progression[bassParameters.beat/4] + transposition +
                compChords[drum.x1ts(0,2)][i] + (drum.y1ts(0,2)-1) * 12;
         int velocity = drum.w1ts(40,127);
         comp[i].play(note, velocity);
      }
   }
}

void stick2trig(void) { 
   if (bassParameters.status == OFF) turnBassOn();
}

void b14plustrig(void) { 
   cout << "B14+ triggered\n"; 
   bassParameters.status = OFF;
   bassParameters.beat = 0;
   bass.off();
   improv.off();
   comp[0].off();
   bassline.reset();
   commandmode(); 
}

void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }


/*---------------- Poll response functions -------------------------------*/


void stick1pollresponse(void) { 
   if (drum.z1p < 50) comp[0].off();
}

void stick2pollresponse(void) { 
   if (bassParameters.status == OFF) return;
   if (drum.z2p < zlevel) {
      improv.off();
      return;
   }
   if (drum.cx2p(0, 5) || drum.cy2p(0,2) || (drum.z2p >= zlevel &&
           drum.z2po < zlevel)) {
      improv.play(48 + 12 * drum.y2ps(0, 2) + BluesScale[drum.x2ps(0,5)] + 
                  transposition, drum.p2p);
   }
}

void potspollresponse(void) { }


/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

