//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon May 13 18:19:14 PDT 1996
// Last Modified: Mon May 13 18:19:24 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/drumloop.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

EventBuffer eventBuffer(100);
NoteLoop drumloop;

/***************** Beginning of user functions ****************************/

class LoopParameters {
   public:
      int status;
      LoopParameters(void) { 
           status = 0;
      }
};

LoopParameters loopParameters;

#define PARAMS ((LoopParameters*)((FunctionNote*)note)->parameters())

static void loopingFunction(Note * note) {
   static Note tempnote;
   if (PARAMS->status == -1) {
      note->off();
   } else if (PARAMS->status == 0) {
      note->setOnTime(t_time + 1000);
   } else {
      midi.play(CH_10, drumloop.nextKey(), drumloop.nextVel());
      note->setOnTime(note->getOnTime() + (int)(drumloop.nextDur() * 
         drum.p3ps()) );
   }
}

/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">    x1t:                                                      \n"
   << ">    y1t:                                                      \n"
   << ">    w1t:                                                      \n"
   << ">    x1p:                                                      \n"
   << ">    y1p:                                                      \n"
   << ">    z1p:                                                      \n"
   << "> Baton 2:                                                     \n"
   << ">    x2t:                                                      \n"
   << ">    y2t:                                                      \n"
   << ">    w2t:                                                      \n"
   << ">    x2p:                                                      \n"
   << ">    y2p:                                                      \n"
   << ">    z2p:                                                      \n"
   << ">                                                              \n"
   << ">    p1p:                                                      \n"
   << ">    p2p:                                                      \n"
   << ">    p3p:                                                      \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization functions ------------------------------*/

void makedrumloop(void) {
   
   char keys[12] = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
   char vels[12] = {0};
   long durs[12] = {300, 300, 300, 300, 300, 300, 300, 300, 300, 300, 300, 300};

   drumloop.setKeys(keys, 12);
   drumloop.setVels(vels, 12);
   drumloop.setDurs(durs, 12);

   cout << " A dur is: " << drumloop.nextDur() << endl;
}

void initialization(void) { 
   description(); 
   makedrumloop();
   drum.setPollRate(10);
   eventBuffer.setPollRate(10);
   FunctionNote tempnote;
   tempnote.setFunction(&loopingFunction);
   tempnote.setParameters(&loopParameters);
   eventBuffer.insert(&tempnote);
}

/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { 
   eventBuffer.checkPoll();
   drum.checkPollPots();
}

/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { 
   if (loopParameters.status == 0) loopParameters.status = 1;
   char newnote = drum.x1ts(C3,C4);
   drumloop.setCurKey(newnote);
   drumloop.setCurVel(drum.w1t);
   midi.play(CH_10, newnote, drum.w1t);
}

void stick2trig(void) { 
   cout << "Baton 2 triggered\n"; 
}
void b14plustrig(void) { 
   cout << "B14+ triggered\n"; 
   loopParameters.status = 0;
   commandmode(); 
}

void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

