// guitar-simple.cc -- Simulate a virtual guitar on the radio drum.  Idea
//	taken from a fellow students final project in Music 120, Winter 1996.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Tue Apr  9 21:27:50 PDT 1996
// Last Modified: Tue Apr  9 21:27:59 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/guitar.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include <math.h>				 // for abs function
#include "main.cc"
#include "psr-520.h"     			 // Timbre Names for PSR-520

enum strings {string1 = 3, string2 = 5, string3 = 7, string4 = 9, 
              string5 = 11, string6 = 13};


/***************** Beginning of user function *****************************/

Note  st1(CH_1, PSR_ACOUSTIC_GUITAR_NYLON);        // Bass   E string
Note  st2(CH_2, PSR_ACOUSTIC_GUITAR_NYLON);        //        A string
Note  st3(CH_3, PSR_ACOUSTIC_GUITAR_NYLON);        //        D string
Note  st4(CH_4, PSR_ACOUSTIC_GUITAR_NYLON);        //        G string
Note  st5(CH_5, PSR_ACOUSTIC_GUITAR_NYLON);        //        B string
Note  st6(CH_6, PSR_ACOUSTIC_GUITAR_NYLON);        // Treble E string

int xpos1 = 0;		// position of baton 1 in the y-axis
int ypos2 = 0;		// position of baton 2 in the y-axis
int new1Q = 0;		// boolean for if baton 1 in new box
int new2Q = 0;		// boolean for if baton 2 in new box
int test = 0;


/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Basic idea for a radio baton guitar.  Only open strings  |\n"
   << "| on this example.                                            |\n"
   << "|                          -- Craig Stuart Sapp, 10 Apr 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">    x1p:                                                      \n"
   << ">    y1p:                                                      \n"
   << ">    z1p:                                                      \n"
   << "> Baton 2:                                                     \n"
   << ">   trig:                                                      \n"
   << ">                                                              \n"
   << ">    p1p:                                                      \n"
   << ">    p2p:                                                      \n"
   << ">    p3p:                                                      \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization algorithms -----------------------------*/

void initialization(void) { 
   description();
   drum.setPollRate(10);       // poll rate of radio drum set to 10 milliseconds
   st1.sendTimbre(); st1.setKeyno(E2);
   st2.sendTimbre(); st2.setKeyno(A2);
   st3.sendTimbre(); st3.setKeyno(D3);
   st4.sendTimbre(); st4.setKeyno(G3);
   st5.sendTimbre(); st5.setKeyno(B3);
   st6.sendTimbre(); st6.setKeyno(E4);
}

/*---------------- Main loop algorithms ----------------------------------*/

void mainloopalgorithms(void) { 
   drum.checkPollBatons();

   // if baton 1 moves onto a new string:
   if (abs(new1Q) >= 2 && (xpos1 % 2) == 1) {
      switch (xpos1) {
         case string1:  st1.setVelocity(drum.z1p); st1.play(); break;
         case string2:  st2.setVelocity(drum.z1p); st2.play(); break;
         case string3:  st3.setVelocity(drum.z1p); st3.play(); break;
         case string4:  st4.setVelocity(drum.z1p); st4.play(); break;
         case string5:  st5.setVelocity(drum.z1p); st5.play(); break;
         case string6: st6.setVelocity(drum.z1p); st6.play(); break;
      }
      new1Q = OFF;
//      cout << "playing string " << xpos1 << endl;
   }
//   if (test) {
//      test = 0;
//      cout << "\t\t\tBox is " << xpos1 << " x1p is " << drum.x1p 
//           << " new1Q is " << new1Q
//           << endl;
//   }
      
}
/*---------------- Trigger response functions ----------------------------*/

void stick1trig(void) { }

void stick2trig(void) { }

void b14plustrig(void) { 
   st1.off(); st2.off(); st3.off(); st4.off(); st5.off(); st6.off();
   commandmode(); 
}

void b15plustrig(void) { programhelp(); }

/*---------------- Poll response algorithms ------------------------------*/

void stick1pollresponse(void) { 
   xpos1 = drum.x1ps(0,15); 
   test = drum.cx1p(0,15);
   new1Q += drum.cx1p(0,15);
   if (drum.z1p < 50) {
      new1Q = 0;
   }
}

void stick2pollresponse(void) { }
void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

