// hundred.cc -- Test of simultaneous triggering
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Mar  9 00:43:13 PST 1996
// Last Modified: Tue Apr 23 00:04:22 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/hundred.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "/user/c/craig/bin/smake FILENAME" to compile this file.
//

#include "main.cc"

Note voice;

/***************** Beginning of user functions ****************************/

int makeDigit(int x, int y) {
   static const int digitHolder[5][2] =                     // 1 2 3 4 5
                           {6,1,7,2,8,3,9,4,0,5};           // 6 7 8 9 0
   return digitHolder[x][y];
}

int makeKeyno(void) {
   return  10 * makeDigit(drum.x1ts(0,4), drum.y1ts(0,1)) +
            1 * makeDigit(drum.x2ts(0,4), drum.y2ts(0,1));
}



/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|    Test of the detection of simultaneous triggering         |\n"
   << "|    of both batons.  Baton divided into 10 locations:        |\n"
   << "|    5 divisions along the x direction and 2 divisions along  |\n"
   << "|    the y direction.  Batons  control the key number of the  |\n"
   << "|    MIDI note: Baton 1 controlling the tens digits of the    |\n"
   << "|    key number, and Baton 2 controlling the ones digit of    |\n"
   << "|    the key number. Digits are arranged on the radio drum as:|\n"
   << "|              1  2  3  4  5                                  |\n"
   << "|              6  7  8  9  0                                  |\n"
   << "|                           -- Craig Stuart Sapp, 9 Mar 1996  |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1: Tens digit of keyno to play                         \n"
   << ">    x1t: Five divisions                                       \n"
   << ">    y1t: Two divisions                                        \n"
   << ">    w1t: Averages with w2t for key velocity of note.          \n"
   << "> Baton 2: Ones digit of keyno to play                         \n"
   << ">    x2t: Five divisions                                       \n"
   << ">    y2t: Two divisions                                        \n"
   << ">    w2t: Averages with w1t for key velocity of note.          \n"
   << ">                                                              \n"
   << ">    p1p: Channel Number to play note on.                      \n"
   << ">    p2p: Timber of note.                                      \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   description(); 
   drum.setPollRate(20);	// poll rate of radio drum set to 20 millisec
}

/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { drum.checkPollPots(); }

/*---------------- Trigger response functions ----------------------------*/

void ActIfTogether(void) {
   if (drum.togetherQ()) {
      cout << "Keyno = " << makeKeyno() << endl;
      voice.play(makeKeyno(), 
                 (drum.w1t+drum.w2t)/2, 
                 drum.p1ps(0, 15), 
                 drum.p2ps(0,63));
      drum.clearTriggers();
   }
}

void stick1trig(void) {
   cout << "Baton 1 triggered. " << endl;
   drum.setT1(t_time);
   ActIfTogether();
}

void stick2trig(void) {
   cout << "Baton 2 triggered. " << endl;
   drum.setT2(t_time);
   ActIfTogether();
}


void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { 
   if (drum.cp1p(1,16)) {
      cout << "\t\t\t Channel = " << drum.p1ps(1,16) << endl;
   }

   if (drum.cp2p(0,63)) {
      cout << "\t    Timbre = " << drum.p2ps(0,63) << endl;
   }
}

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

