//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed May 15 22:54:21 PDT 1996
// Last Modified: Wed May 15 22:54:28 PDT 1996
// Filename:      /user/c/craig/lang/c/drum/examples/pad.cc
// Syntax:        C++ (Gnu C++ compiler: /Local/Unix/bin/g++)
// $Smake:        g++ -O -o %b %f -I../include -L../lib %
//                -ldrum -lmidi -loptions && strip %b
//                Type "../bin/smake FILENAME" to compile this file.
//

#include "main.cc"

EventBuffer eventBuffer(2000);

class PadParameters;
typedef void (*padFuncPtr)(PadParameters* params);

class PadParameters {
   public:
      int status;
      FunctionNote* eventPtr;
      padFuncPtr actionFunction;
      PadParameters(void) { status = 0; };
};


PadParameters padParameters[2][2];
   

/***************** Beginning of user functions ****************************/

#define PARAMS ((PadParameters*)((FunctionNote*)note)->parameters())

static void padFunction(Note* note) {
   if (PARAMS->status == -1) {
      note->off();
   } else if (PARAMS->status == 1) {
      PARAMS->actionFunction(PARAMS);
   }
}

static void action00(PadParameters* params) {
   midi.play(CH_10, C3, drum.x1p);
   params->eventPtr->setOnTime(t_time+drum.x2p+200);
}

static void action01(PadParameters* params) {
   midi.play(CH_10, F3, drum.y1p);
   params->eventPtr->setOnTime(t_time+drum.y2p+200);
}

static void action10(PadParameters* params) {
   midi.play(CH_10, GS3, drum.x2p);
   params->eventPtr->setOnTime(t_time+drum.x1p+200);
}

static void action11(PadParameters* params) {
   midi.play(CH_10, B3, drum.y2p);
   params->eventPtr->setOnTime(t_time+drum.y1p+200);
}


/*---------------- Documentation functions -------------------------------*/

void description(void) {
   cout
   << "______________________________________________________________ \n"
   << "| Description:                                                |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "|                                                             |\n"
   << "---------------------------------------------------------------\n"
   ;
}


void programhelp(void) {
   cout
   << ">                                                              \n"
   << "> Baton 1:                                                     \n"
   << ">    x1t:                                                      \n"
   << ">    y1t:                                                      \n"
   << ">    w1t:                                                      \n"
   << ">    x1p:                                                      \n"
   << ">    y1p:                                                      \n"
   << ">    z1p:                                                      \n"
   << "> Baton 2:                                                     \n"
   << ">    x2t:                                                      \n"
   << ">    y2t:                                                      \n"
   << ">    w2t:                                                      \n"
   << ">    x2p:                                                      \n"
   << ">    y2p:                                                      \n"
   << ">    z2p:                                                      \n"
   << ">                                                              \n"
   << ">    p1p:                                                      \n"
   << ">    p2p:                                                      \n"
   << ">    p3p:                                                      \n"
   << ">   +b14: Command mode.                                        \n"
   << ">   +b15: This help screen.                                    \n"
   << ">                                                              \n"
   ;
}

/*---------------- Initialization functions ------------------------------*/

void initialization(void) { 
   description(); 
   eventBuffer.setPollRate(10);
   drum.setPollRate(30);

   // insert pads into eventBuffer:
   for (int i=0; i<2; i++) {
      for (int j=0; j<2; j++) {
         FunctionNote tempnote;
         tempnote.setFunction(&padFunction);
         tempnote.setParameters(&padParameters[i][j]);
         padParameters[i][j].eventPtr = 
            (FunctionNote*)eventBuffer.insert(&tempnote);
      }
   }
   padParameters[0][0].actionFunction = &action00;
   padParameters[0][1].actionFunction = &action01;
   padParameters[1][0].actionFunction = &action10;
   padParameters[1][1].actionFunction = &action11;
}

/*---------------- Main loop functions -----------------------------------*/

void mainloopalgorithms(void) { 
   eventBuffer.checkPoll();
   drum.checkPollBatons();
}

/*---------------- Trigger response functions ----------------------------*/

void ActIfTogether(void) {
   if (!drum.togetherQ(60)) return;
   int val1, val2;
   int val3, val4;

   val1 = drum.x1ts(0,1);
   val2 = drum.y1ts(0,1);
   val3 = drum.x2ts(0,1);
   val4 = drum.y2ts(0,1);

   if (val1 == val3 && val2 == val4) {
      padParameters[val1][val2].status = !padParameters[val1][val2].status;
   }

   drum.clearTriggers();
}


void stick1trig(void) { 
   drum.setT1(t_time);
   ActIfTogether();
}

void stick2trig(void) { 
   drum.setT2(t_time);
   ActIfTogether();
} 

void b14plustrig(void) { cout << "B14+ triggered\n"; commandmode(); }
void b15plustrig(void) { cout << "B15+ triggered\n"; programhelp(); }

/*---------------- Poll response functions -------------------------------*/

void stick1pollresponse(void) { }
void stick2pollresponse(void) { }
void potspollresponse(void) { }

/*---------------- Actions to take just before leaving program -----------*/

void cleanup(void) { }

/***************** End of user functions **********************************/

