//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 23 December 1997
// Last Modified: 23 December 1997
// Filename:      print2.cpp
// Syntax:        Visual C++ 5.0; Radio Baton Improv
//  
// Description: Prints a message to the screen when a button, baton
//      or foot pedal is pressed.  Also controls the speed of printing
//	statements in mainloopalgorithm
//

#include "improv.h" 

#define QUANTIZATION 100

/*----------------- beginning of improvization algorithms ---------------*/

long nextprinttime;   // time to print the statement in mainloopalgorithm
int  increment;       // time between print statements in mainloopalgorithm

/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << "print --\n";
   cout << "   This program prints a statement whenever a stick, button\n";
   cout << "   or footpedal is triggered.\n";
   cout << "\n";
   cout << "   The two plus buttons control the speed of print statements\n";
   cout << "   in mainloopalgorithm:\n";
   cout << "      b14+ button will increase print time by 10 milliseconds\n";
   cout << "      b15+ button will decrease print time by 10 milliseconds\n";
   cout << "         (down to a minimum of 100 milliseconds)\n";
   cout << endl;
} 

void initialization(void) { 
   cout << "In initialization function" << endl; 
   nextprinttime = 0;
   increment = 2000;
}

void finishup(void) { 
   cout << "In cleanup function" << endl; 
}


/*-------------------- main loop algorithms -----------------------------*/

void mainloopalgorithms(void) { 
   if (t_time >= nextprinttime) {
      cout << "In main loop algorithm" << endl;
      nextprinttime = t_time + increment;       // set next print time
   }
}


/*-------------------- triggered algorithms -----------------------------*/

void stick1trig(void) { 
   cout << "\nStick 1 triggered" << endl; 
}

void stick2trig(void) { 
   cout << "\nStick 2 triggered" << endl; 
}

void b14plustrig(void) { 
   cout << "\nb14+ button pushed" << endl; 
   increment += QUANTIZATION;
}

void b15plustrig(void) { 
   cout << "\nb15+ button pushed" << endl; 
   increment -= QUANTIZATION;
   if (increment < 100)  increment = 100;
}

void b14minusuptrig(void) { 
   cout << "\nb14- pedal depressed" << endl; 
}

void b14minusdowntrig(void) { 
   cout << "\nb14- pedal released" << endl; 
}

void b15minusuptrig(void) { 
   cout << "\nb15- pedal depressed" << endl; 
}

void b15minusdowntrig(void) { 
   cout << "\nb15- pedal released" << endl; 
}

void keyboardchar(int key) { 
   cout << "A character was received by keyboardchar: " << key << endl;
}


/*------------------ end improvization algorithms -----------------------*/


/*
   some functions and variables provided by the support program

   program_change(channel, instrument); -------- sets the timbre for a channel
   control_change(channel, controller, value); - sends a continuous controller 
   note_on(channel, keynumber, keyvelocity); --- plays a MIDI note
   note_off(channel, keynumber); --------------- same as note_on with 0 vel.
   raw_send(channel, command, data1, data2); --- send some midi command
   long t_time; -------------------------------- current time in milliseconds
   short pt1, pt2, pt3, pt4; ------------------- current pot (dial) positions
   short xt1,yt1,zt1,xt2,yt2,zt2; -------------- current stick positions
   short wh1, wh2; ----------------------------- whack strength of a trigger
   short xw1, yw1, xw2, yw2; ------------------- x and y positions at trigger
   short hit1, set1, hit2, set2; --------------- hit and set levels of sticks
   short whack1,whack2; ------------------------ registers set by stick triggers
   long  trigtime1, trigtime2; ----------------- times of stick triggers
*/


